<?php

class CorreoArgentinoMiCorreoService implements CorreoArgentinoServiceMiCorreoInterface
{
    public $client;
    public $url;
    private $username;
    private $password;
    private $authHash;
    private $accessToken;

    public function __construct()
    {
        $credentials = maybe_unserialize(get_option(CA_CREDENTIALS));
        $this->url = (int)$credentials['sandbox'] === 1 ? CA_MI_CORREO_API_SANDBOX_URL : CA_MI_CORREO_API_URL;
        $this->authHash = $credentials['auth_hash'];
        if (isset($credentials['access_token']) && !$this->isValidToken()) {
            $this->renewToken();
        }
        $this->accessToken = $credentials['access_token'];
    }

    /**
     * @throws Exception
     */
    public static function isValidToken(): bool
    {
        $credentials = maybe_unserialize(get_option(CA_CREDENTIALS));
        if (!isset($credentials['access_token']) || !isset($credentials['expire'])) {
            return false;
        }
        $now = new DateTime('now');
        $expires = new DateTime($credentials['expire']);
        if ($expires < $now) {
            return false;
        }
        return true;
    }

    /**
     * @return bool
     */
    public function renewToken(): bool
    {
        $response = $this->login();
        $credentials = Utils::getCredentials();
        if ($response && isset($response['token'])) {
            $credentials["access_token"] = $response["token"];
            $credentials["expire"] = $response["expire"];
            $data = maybe_serialize($credentials);
            update_option(CA_CREDENTIALS, $data);
            return true;
        }
        return false;
    }

    public function login()
    {
        $headers = $this->setHeaders([
            "Authorization" => "Basic " . $this->authHash
        ]);

        $request = wp_remote_post(
            $this->url . '/token',
            [
                'method' => 'POST',
                'headers' => $headers,
            ]
        );

        $response = json_decode(wp_remote_retrieve_body($request), true);
        if (wp_remote_retrieve_response_code($request) != 200) {
            return $response;
        }
        return $response;
    }

    public function setHeaders($headers = [])
    {
        $defaultHeaders = [
            "Content-Type" => "application/json",
            "Accept" => "application/json",
            "Connection" => "keep-alive",
        ];
        return array_merge($defaultHeaders, $headers);
    }

    public function registerOrder($orderId)
    {
        $order = wc_get_order($orderId);
        $orderData = $order->get_data();
        $shippingData = $orderData['shipping'];
        $billing = $orderData['billing'];
        $branchId = $order->get_meta(CA_CHOSEN_BRANCH);
        $shippingMethods = $order->get_shipping_methods();
        $chosenShippingMethod = reset($shippingMethods);
        $productType = $chosenShippingMethod->get_meta(CA_CHOSEN_PRODUCT_TYPE);
        $items = $order->get_items();
        $shippingMethod = $order->get_shipping_method();

        $isBranch = stripos($shippingMethod, BRANCH);
        $settings = Utils::getSettings();

        $recipient = [
            "name" => $billing['first_name'] . ' ' . $billing['last_name'],
            "email" => $billing['email'],
            "phone" => $shippingData['phone'],
            "cellPhone" => $shippingData['phone'],
        ];

        $maxVolumetricWeight = 0;

        foreach ($items as $item) {
            $product = $item->get_product();

            $length = floatval($product->get_length());
            $width = floatval($product->get_width());
            $height = floatval($product->get_height());

            $volumetricWeight = ($length * $width * $height);

            $maxVolumetricWeight = max($maxVolumetricWeight, $volumetricWeight);
        }


        $filteredItems = array_filter($items, function ($item) use ($maxVolumetricWeight) {
            $itemVolumetricWeight = (float)$item->get_product()->get_length() * (float)$item->get_product()->get_width() * (float)$item->get_product()->get_height();
            return $itemVolumetricWeight == $maxVolumetricWeight;
        });

        $declaredValueTotal = array_reduce($filteredItems, function ($carry, $item) {
            $carry += (float)$item->get_total();
            return $carry;
        });

        $heightTotal = array_reduce($filteredItems, function ($carry, $item) {
            $carry += (float)$item->get_product()->get_height();
            return $carry;
        });

        $lengthTotal = array_reduce($filteredItems, function ($carry, $item) {
            $carry += (float)$item->get_product()->get_length();
            return $carry;
        });

        $widthTotal = array_reduce($filteredItems, function ($carry, $item) {
            $carry += (float)$item->get_product()->get_width();
            return $carry;
        });

        $productFiltered = array_reduce($filteredItems, function ($carry, $item) {
            return $item;
        });

        $productId = $productFiltered['product_id'];
        $product = wc_get_product($productId);
        $productWeight = (float)$product->get_weight();


        $shipping = [
            "deliveryType" => $isBranch ? "S" : "D",
            "productType" => $productType,
            "declaredValue" => $declaredValueTotal,
            "weight" => Utils::fromKgToGrams($productWeight),
            "height" => Utils::toInteger($heightTotal),
            "length" => Utils::toInteger($lengthTotal),
            "width" => Utils::toInteger($widthTotal),
            "agency" => "$branchId",
            "address" => [
                "streetName" => $shippingData['address_1'],
                "streetNumber" => $shippingData['address_2'],
                "floor" => $shippingData['floor'],
                "apartment" => $shippingData["department"],
                "city" => $shippingData['city'],
                "provinceCode" => $shippingData['state'],
                "postalCode" => Utils::normalizeZipCode($shippingData['postcode'])
            ],
        ];

        $sender = [
            "name" => $settings['first_name'] . " " . $settings['last_name'],
            "phone" => Utils::cleanPhone($settings['phone']),
            "cellPhone" => Utils::cleanPhone($settings['cellphone']),
            "email" => $settings['email'],
            "originAddress" => [
                "streetName" => $settings['street_name'],
                "streetNumber" => $settings['street_number'],
                "floor" => $settings['floor'],
                "apartment" => $settings['department'],
                "city" => $settings['city_name'],
                "provinceCode" => $settings['state'],
                "postalCode" => Utils::normalizeZipCode($settings['zip_code'])
            ]
        ];

        $extOrderId = $orderId;
        $body = [
            "customerId" => $settings['customer_id'],
            "extOrderId" => strval($extOrderId),
            "orderNumber" => $orderId,
            "sender" => $sender,
            "recipient" => $recipient,
            "shipping" => $shipping
        ];

        $headers = $this->setHeaders([
            "Authorization" => "Bearer " . $this->accessToken,
            "Content-Type" => "application/json"
        ]);

        $request = wp_remote_post(
            $this->url . '/shipping/import',
            [
                'method' => 'POST',
                'headers' => $headers,
                'body' => json_encode($body),
            ]
        );

        $response = json_decode(wp_remote_retrieve_body($request), true);
        $response["status"] = wp_remote_retrieve_response_code($request);
        if ($response && $response["status"] == 200) {
            $response['reference'] = $extOrderId;
            $order->update_meta_data(CA_TRACKING_NUMBER, $extOrderId);
            $order->save();
        }else {
            $response['api_error_message'] = $response['message'];
        }

        return $response;

    }

    /**
     * @throws Exception
     */
    public function getRates($postalCode, $deliveryType, $dimensions)
    {
        $settings = Utils::getSettings();
        if (!isset($this->accessToken)) {
            throw new Exception('access token no válido o indefinido');
        }

        $headers = $this->setHeaders([
            "Authorization" => "Bearer " . $this->accessToken
        ]);

        $body = [
            "customerId" => $settings["customer_id"],
            "postalCodeOrigin" => Utils::normalizeZipCode($settings["zip_code"]),
            "postalCodeDestination" => Utils::normalizeZipCode($postalCode),
            "deliveredType" => $deliveryType,
            "dimensions" => $dimensions
        ];

        $request = wp_safe_remote_post(
            $this->url . '/rates',
            [
                'method' => 'POST',
                'headers' => $headers,
                'body' => json_encode($body)
            ]
        );

        $response = json_decode(wp_remote_retrieve_body($request), true);
        $response['status'] = wp_remote_retrieve_response_code($request);
        $response['message'] = wp_remote_retrieve_response_message($request);
        return $response;
    }

    public function getBranches($query = [])
    {
        $defaultQuery = ["services" => 'pickup_availability'];
        $query = array_merge($defaultQuery, $query);
        $headers = $this->setHeaders([
            "Authorization" => "Bearer " . $this->accessToken
        ]);
        $request = wp_remote_get(
            add_query_arg($query, $this->url . '/agencies'),
            [
                'method' => 'GET',
                'headers' => $headers
            ]
        );
        $response['status'] = wp_remote_retrieve_response_code($request);
        $response['message'] = wp_remote_retrieve_response_message($request);
        $response = json_decode(wp_remote_retrieve_body($request), true);
        return $response;

    }

    public function createAccount($body)
    {
        $headers = $this->setHeaders([
            "Authorization" => "Bearer " . $this->accessToken,
            "Content-Type" => "application/json"
        ]);

        $request = wp_remote_post(
            $this->url . '/register?t=' . time(),
            [
                'method' => 'POST',
                'headers' => $headers,
                'body' => json_encode($body),
            ]
        );
        $response = json_decode(wp_remote_retrieve_body($request), true);
        $response["status"] = wp_remote_retrieve_response_code($request);
        $response["message"] = wp_remote_retrieve_response_message($request);
        return $response;
    }

    public function userValidate($email, $password)
    {

        $headers = $this->setHeaders([
            "Authorization" => "Bearer " . $this->accessToken
        ]);

        $request = wp_remote_post(
            $this->url . '/users/validate',
            [
                'method' => 'POST',
                'headers' => $headers,
                'body' => json_encode(["email" => $email, "password" => $password])
            ]
        );

        $response = json_decode(wp_remote_retrieve_body($request), true);
        $response["status"] = wp_remote_retrieve_response_code($request);
        $response["message"] = wp_remote_retrieve_response_message($request);
        return $response;


    }
}